@@ -4,21 +4,18 @@
    Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
    Created: October 1998
 
-   This file is part of the GNUstep Base Library.
+   This file is part of the GNUstep Project
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+    
+   You should have received a copy of the GNU General Public  
+   License along with this library; see the file COPYING.LIB.
+   If not, write to the Free Software Foundation,
+   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */
 
 #include	<Foundation/NSObject.h>
@@ -34,6 +31,8 @@
 #include	<Foundation/NSUserDefaults.h>
 #include	<Foundation/NSDistributedNotificationCenter.h>
 
+#include	<unistd.h>
+
 #include	"gdnc.h"
 
 @interface	GDNCNotification : NSObject
@@ -177,15 +176,16 @@
 
 - (void) dealloc
 {
+  NSNotificationCenter	*nc = [NSNotificationCenter defaultCenter];
   NSMapEnumerator	enumerator;
   NSConnection		*connection;
   NSMapTable		*clients;
 
   if (conn)
     {
-      [NSNotificationCenter removeObserver: self
-				      name: NSConnectionDidDieNotification
-				    object: conn];
+      [nc removeObserver: self
+		    name: NSConnectionDidDieNotification
+		  object: conn];
       [conn release];
       conn = nil;
     }
@@ -198,9 +198,9 @@
   while (NSNextMapEnumeratorPair(&enumerator,
 		(void**)&connection, (void**)&clients) == YES)
     {
-      [NSNotificationCenter removeObserver: self
-				      name: NSConnectionDidDieNotification
-				    object: connection];
+      [nc removeObserver: self
+		    name: NSConnectionDidDieNotification
+		  object: connection];
       [self removeObserversForClients: clients];
       NSFreeMapTable(clients);
     }
@@ -239,10 +239,11 @@
    *	Get notifications for new connections and connection losses.
    */
   [conn setDelegate: self];
-  [NSNotificationCenter addObserver: self
-			   selector: @selector(connectionBecameInvalid:)
-			       name: NSConnectionDidDieNotification
-			     object: conn];
+  [[NSNotificationCenter defaultCenter]
+    addObserver: self
+       selector: @selector(connectionBecameInvalid:)
+	   name: NSConnectionDidDieNotification
+	 object: conn];
   return self;
 }
 
@@ -344,10 +345,11 @@
 {
   NSMapTable	*table;
 
-  [NSNotificationCenter addObserver: self
-			   selector: @selector(connectionBecameInvalid:)
-			       name: NSConnectionDidDieNotification
-			     object: newConn];
+  [[NSNotificationCenter defaultCenter]
+    addObserver: self
+       selector: @selector(connectionBecameInvalid:)
+	   name: NSConnectionDidDieNotification
+	 object: newConn];
   [newConn setDelegate: self];
   /*
    *	Create a new map table entry for this connection with a value that
@@ -364,9 +366,10 @@
 {
   id connection = [notification object];
 
-  [NSNotificationCenter removeObserver: self
-				  name: NSConnectionDidDieNotification
-				object: connection];
+  [[NSNotificationCenter defaultCenter]
+    removeObserver: self
+	      name: NSConnectionDidDieNotification
+	    object: connection];
 
   if (connection == conn)
     {
@@ -389,6 +392,7 @@
 	}
       NSMapRemove(connections, connection);
     }
+  return nil;
 }
 
 - (void) registerClient: (id<GDNCClient>)client
