@@ -40,11 +40,13 @@
 #include <Foundation/NSConnection.h>
 #include <Foundation/NSDistantObject.h>
 #include <Foundation/NSMethodSignature.h>
+#include <Foundation/NSPathUtilities.h>
 #include <Foundation/NSUserDefaults.h>
 #include <Foundation/NSSerialization.h>
 #include <Foundation/NSPortNameServer.h>
+#include <Foundation/NSTask.h>
 
-#include <Foundation/fast.x>
+#include <base/fast.x>
 
 #include <AppKit/NSApplication.h>
 #include <AppKit/NSPasteboard.h>
@@ -56,19 +58,16 @@
 
 #include <AppKit/GSServicesManager.h>
 
-#define stringify_it(X) #X
-#define prog_path(X,Y) \
-  stringify_it(X) "/Tools/" GNUSTEP_TARGET_DIR "/" LIBRARY_COMBO
-
 /*
- *	The GNUListener class is for talking to other applications.
+ *	The GSListener class is for talking to other applications.
  *	It is a proxy with some dangerous methods implemented in a
  *	harmless manner to reduce the chances of a malicious app
- *	messing with us.
+ *	messing with us.  This is responsible for forwarding service
+ *      requests and other communications with the outside world.
  */
-@interface      GNUListener : NSObject
+@interface      GSListener : NSObject
 + (id) connectionBecameInvalid: (NSNotification*)notification;
-+ (GNUListener*) listener;
++ (GSListener*) listener;
 + (id) servicesProvider;
 + (void) setServicesProvider: (id)anObject;
 - (Class) class;
@@ -76,10 +75,22 @@
 - (void) release;
 - (id) retain;
 - (id) self;
+- (BOOL) application: (NSApplication*)theApp
+	    openFile: (NSString*)file;
+- (BOOL) application: (NSApplication*)theApp
+   openFileWithoutUI: (NSString*)file;
+- (BOOL) application: (NSApplication*)theApp
+	openTempFile: (NSString*)file;
+- (BOOL) application: (NSApplication*)theApp
+	   printFile: (NSString*)file;
+- (void) performService: (NSString*)name
+	 withPasteboard: (NSPasteboard*)pb
+	       userData: (NSString*)ud
+		  error: (NSString**)e;
 @end
 
 static NSConnection	*listenerConnection = nil;
-static GNUListener	*listener = nil;
+static GSListener	*listener = nil;
 static id		servicesProvider = nil;
 
 void
@@ -92,9 +103,10 @@ NSUnregisterServicesProvider(NSString *name)
        *        the given port name.
        */
       [[NSPortNameServer defaultPortNameServer] removePortForName: name];
-      [NSNotificationCenter removeObserver: [GNUListener class]
-                                      name: NSConnectionDidDieNotification
-                                    object: listenerConnection];
+      [[NSNotificationCenter defaultCenter]
+	removeObserver: [GSListener class]
+		  name: NSConnectionDidDieNotification
+		object: listenerConnection];
       [listenerConnection release];
       listenerConnection = nil;
     }
@@ -111,51 +123,56 @@ NSRegisterServicesProvider(id provider, NSString *name)
        *	the given port name.
        */
       [[NSPortNameServer defaultPortNameServer] removePortForName: name];
-      [NSNotificationCenter removeObserver: [GNUListener class]
-				      name: NSConnectionDidDieNotification
-				    object: listenerConnection];
+      [[NSNotificationCenter defaultCenter]
+	removeObserver: [GSListener class]
+		  name: NSConnectionDidDieNotification
+		object: listenerConnection];
       [listenerConnection release];
       listenerConnection = nil;
     }
   if (name && provider)
     {
       listenerConnection = [NSConnection newRegisteringAtName: name
-				       withRootObject: [GNUListener listener]];
+				       withRootObject: [GSListener listener]];
       if (listenerConnection)
 	{
 	  [listenerConnection retain];
-	  [NotificationDispatcher
-                    addObserver: [GNUListener class]
+	  [[NSNotificationCenter defaultCenter]
+                    addObserver: [GSListener class]
 		       selector: @selector(connectionBecameInvalid:)
 			   name: NSConnectionDidDieNotification
 			 object: listenerConnection];
 	}
+      else
+       [NSException raise: NSGenericException
+		   format: @"unable to register %@", name];
     }
   ASSIGN(servicesProvider, provider);
 }
 
 /*
- *	The GNUListener class exists as a proxy to forward messages to
+ *	The GSListener class exists as a proxy to forward messages to
  *	service provider objects.  It implements very few methods and
  *	those that it does implement are generally designed to defeat
  *	any attack by a malicious program.
  */
-@implementation GNUListener
+@implementation GSListener
 
 + (id) connectionBecameInvalid: (NSNotification*)notification
 {
   NSAssert(listenerConnection==[notification object],
 	NSInternalInconsistencyException);
 
-  [NSNotificationCenter removeObserver: self
-				  name: NSConnectionDidDieNotification
-				object: listenerConnection];
+  [[NSNotificationCenter defaultCenter]
+    removeObserver: self
+	      name: NSConnectionDidDieNotification
+	    object: listenerConnection];
   [listenerConnection release];
   listenerConnection = nil;
   return self;
 }
 
-+ (GNUListener*) listener
++ (GSListener*) listener
 {
   if (listener == nil)
     {
@@ -175,7 +192,7 @@ NSRegisterServicesProvider(id provider, NSString *name)
 
   if (servicesProvider != anObject)
     {
-      appName = [[[NSProcessInfo processInfo] processName] lastPathComponent];
+      appName = [[NSProcessInfo processInfo] processName];
       NSRegisterServicesProvider(anObject, appName);
     }
 }
@@ -192,6 +209,7 @@ NSRegisterServicesProvider(id provider, NSString *name)
 - forward: (SEL)aSel :(arglist_t)frame
 {
   NSString      *selName = NSStringFromSelector(aSel);
+  id            delegate;
 
   /*
    *    If the selector matches the correct form for a services request,
@@ -201,11 +219,91 @@ NSRegisterServicesProvider(id provider, NSString *name)
   if ([selName hasSuffix: @":userData:error:"])
     return [servicesProvider performv: aSel :frame];
 
+  /*
+   *    If tha applications delegate can handle the message - forward to it.
+   */
+  delegate = [[NSApplication sharedApplication] delegate];
+  if ([delegate respondsToSelector: aSel] == YES)
+    return [delegate performv: aSel :frame];
+
   [NSException raise: NSGenericException
 	      format: @"method %s not implemented", sel_get_name(aSel)];
   return nil;
 }
 
+- (BOOL) application: (NSApplication*)theApp
+	    openFile: (NSString*)file
+{
+  id	del = [NSApp delegate];
+
+  if ([del respondsToSelector: _cmd])
+    return [del application: theApp openFile: file];
+  return NO;
+}
+
+- (BOOL) application: (NSApplication*)theApp
+   openFileWithoutUI: (NSString*)file
+{
+  id	del = [NSApp delegate];
+
+  if ([del respondsToSelector: _cmd])
+    return [del application: theApp openFileWithoutUI: file];
+  return NO;
+}
+
+- (BOOL) application: (NSApplication*)theApp
+	openTempFile: (NSString*)file
+{
+  id	del = [NSApp delegate];
+
+  if ([del respondsToSelector: _cmd])
+    return [del application: theApp openTempFile: file];
+  return NO;
+}
+
+- (BOOL) application: (NSApplication*)theApp
+	   printFile: (NSString*)file
+{
+  id	del = [NSApp delegate];
+
+  if ([del respondsToSelector: _cmd])
+    return [del application: theApp openFile: file];
+  return NO;
+}
+
+- (void) performService: (NSString*)name
+	 withPasteboard: (NSPasteboard*)pb
+	       userData: (NSString*)ud
+		  error: (NSString**)e
+{
+  id	obj = servicesProvider;
+  SEL	msgSel = NSSelectorFromString(name);
+  IMP	msgImp;
+
+  if (obj != nil && [obj respondsToSelector: msgSel])
+    {
+      msgImp = [obj methodForSelector: msgSel];
+      if (msgImp != 0)
+	{
+	  (*msgImp)(obj, msgSel, pb, ud, e);
+	  return;
+	}
+    }
+
+  obj = [[NSApplication sharedApplication] delegate];
+  if (obj != nil && [obj respondsToSelector: msgSel])
+    {
+      msgImp = [obj methodForSelector: msgSel];
+      if (msgImp != 0)
+	{
+	  (*msgImp)(obj, msgSel, pb, ud, e);
+	  return;
+	}
+    }
+
+  *e = @"No object available to provide service"; 
+}
+
 - (void) release
 {
 }
@@ -219,7 +317,7 @@ NSRegisterServicesProvider(id provider, NSString *name)
 {
   return self;
 }
-@end /* GNUListener */
+@end /* GSListener */
 
 
 
@@ -289,18 +387,11 @@ static NSString         *disabledName = @".GNUstepDisabled";
   return manager;
 }
 
-- (void) checkServices
-{
-  system(prog_path(GNUSTEP_INSTALL_PREFIX, "/make_services"));
-
-  [self loadServices];
-}
-
 - (void) dealloc
 {
   NSString          *appName;
 
-  appName = [[[NSProcessInfo processInfo] processName] lastPathComponent];
+  appName = [[NSProcessInfo processInfo] processName];
   [timer invalidate];
   NSUnregisterServicesProvider(appName);
   [languages release];
@@ -330,6 +421,8 @@ static NSString         *disabledName = @".GNUstepDisabled";
   NSWindow      *resp = [[application keyWindow] firstResponder];
   id            obj = nil;
 
+  NSLog(@"doService: called");
+
   for (i = 0; i <= es; i++)
     {
       NSString  *sendType;
@@ -614,7 +707,7 @@ static NSString         *disabledName = @".GNUstepDisabled";
       NSMutableSet      *keyEquivalents;
       unsigned          pos;
       unsigned          loc0;
-      unsigned          loc1;
+      unsigned          loc1 = 0;
       SEL               sel = @selector(doService:);
       NSMenu            *submenu = nil;
 
@@ -717,8 +810,9 @@ static NSString         *disabledName = @".GNUstepDisabled";
               [item setTag: pos];
             }
         }
-      [submenu sizeToFit];
-      [servicesMenu sizeToFit];
+      [submenu update];
+//      [submenu sizeToFit];
+//      [servicesMenu sizeToFit];
       [servicesMenu update];
     }
 }
@@ -728,15 +822,65 @@ static NSString         *disabledName = @".GNUstepDisabled";
  */
 - (void) registerAsServiceProvider
 {
-  NSString          *appName;
+  NSString      *appName;
+  BOOL          registered;
 
-  appName = [[[NSProcessInfo processInfo] processName] lastPathComponent];
+  appName = [[NSProcessInfo processInfo] processName];
   NS_DURING
-    NSRegisterServicesProvider(self, appName);
+    {
+      NSRegisterServicesProvider(self, appName);
+      registered = YES;
+    }
   NS_HANDLER
-    NSLog(@"Warning: Could not access services due to exception: %@\n",
-          [localException reason]);
+    registered = NO;
   NS_ENDHANDLER
+
+  if (registered == NO)
+    {
+      int result = NSRunAlertPanel(appName,
+	@"Application may already be running with this name",
+	@"Continue", @"Abort", @"Rename");
+
+      if (result == NSAlertDefaultReturn || result == NSAlertOtherReturn)
+        {
+          if (result == NSAlertOtherReturn)
+	    appName = [[NSProcessInfo processInfo] globallyUniqueString];
+
+          [[NSPortNameServer defaultPortNameServer] removePortForName: appName];
+
+          NS_DURING
+            {
+              NSRegisterServicesProvider(self, appName);
+              registered = YES;
+            }
+          NS_HANDLER
+            {
+              registered = NO;
+              NSLog(@"Warning: Could not register application due to "
+                    @"exception: %@\n", [localException reason]);
+            }
+          NS_ENDHANDLER
+
+	  /*
+	   *	Something is seriously wrong - we can't talk to the
+	   *	nameserver, so all interaction with the workspace manager
+	   *	and/or other applications will fail.
+	   *	Give the user a chance to keep on going anyway.
+	   */
+	  if (registered == NO)
+	    {
+	      result = NSRunAlertPanel(appName,
+		@"Unable to register application with ANY name",
+		@"Abort", @"Continue", nil);
+
+	      if (result == NSAlertDefaultReturn)
+		registered = YES;
+	    }
+        }
+
+      if (registered == NO)
+        [[NSApplication sharedApplication] terminate: self];
+    }
 }
 
 /*
@@ -797,7 +941,7 @@ static NSString         *disabledName = @".GNUstepDisabled";
 
 - (id) servicesProvider
 {
-  return [GNUListener servicesProvider];
+  return [GSListener servicesProvider];
 }
 
 - (void) setServicesMenu: (NSMenu*)aMenu
@@ -808,7 +952,7 @@ static NSString         *disabledName = @".GNUstepDisabled";
 
 - (void) setServicesProvider: (id)anObject
 {
-  [GNUListener setServicesProvider: anObject];
+  [GSListener setServicesProvider: anObject];
 }
 
 - (int) setShowsServicesMenuItem: (NSString*)item to: (BOOL)enable
@@ -930,7 +1074,6 @@ static NSString         *disabledName = @".GNUstepDisabled";
 {
   if (servicesMenu && [[application mainMenu] autoenablesItems])
     {
-      NSMenuMatrix	*menuCells;
       NSArray   	*a;
       unsigned  	i;
       NSMenu		*mainMenu = [application mainMenu];
@@ -946,8 +1089,7 @@ static NSString         *disabledName = @".GNUstepDisabled";
 	  return;
 	}
 
-      menuCells = [servicesMenu menuCells];
-      a = [menuCells itemArray];
+      a = [servicesMenu itemArray];
 
       for (i = 0; i < [a count]; i++)
         {
@@ -962,8 +1104,7 @@ static NSString         *disabledName = @".GNUstepDisabled";
 	   */
 	  if (title == nil && [[cell target] isKindOfClass: [NSMenu class]])
 	    {
-	      NSMenuMatrix	*subMenuCells = [[cell target] menuCells];
-	      NSArray		*sub = [subMenuCells itemArray];
+	      NSArray		*sub = [[cell target] itemArray];
 	      int		j;
 
 	      shouldBeEnabled = NO;
@@ -981,8 +1122,8 @@ static NSString         *disabledName = @".GNUstepDisabled";
 		  if (subWasEnabled != subShouldBeEnabled)
 		    {
 		      [subCell setEnabled: subShouldBeEnabled];
-		      [subMenuCells setNeedsDisplayInRect:
-				[subMenuCells cellFrameAtRow: j]];
+//		      [subMenuCells setNeedsDisplayInRect:
+//				[subMenuCells cellFrameAtRow: j]];
 		    }
 		}
 	    }
@@ -992,7 +1133,7 @@ static NSString         *disabledName = @".GNUstepDisabled";
           if (wasEnabled != shouldBeEnabled)
             {
               [cell setEnabled: shouldBeEnabled];
-	      [menuCells setNeedsDisplayInRect: [menuCells cellFrameAtRow: i]];
+//	      [menuCells setNeedsDisplayInRect: [menuCells cellFrameAtRow: i]];
             }
         }
     }
@@ -1001,6 +1142,59 @@ static NSString         *disabledName = @".GNUstepDisabled";
 @end /* GSServicesManager */
 
 
+id
+GSContactApplication(NSString *appName, NSString *port, NSDate *expire)
+{
+  id		app;
+
+  NS_DURING
+    {
+      app = [NSConnection rootProxyForConnectionWithRegisteredName: port  
+                                                              host: @""];
+    }
+  NS_HANDLER
+    {
+      return nil;                /* Fatal error in DO    */
+    }
+  NS_ENDHANDLER
+
+  if (app == nil)
+    {
+      if ([[NSWorkspace sharedWorkspace] launchApplication: appName] == NO)
+	{
+	  return nil;		/* Unable to launch.	*/
+	}
+
+      NS_DURING
+	{
+	  app = [NSConnection
+			rootProxyForConnectionWithRegisteredName: port  
+							    host: @""];
+	  while (app == nil && [expire timeIntervalSinceNow] > 0.1)
+	    {
+	      NSRunLoop	*loop = [NSRunLoop currentRunLoop];
+	      NSDate	*next;
+
+	      [NSTimer scheduledTimerWithTimeInterval: 0.1
+					   invocation: nil
+					      repeats: NO];
+	      next = [NSDate dateWithTimeIntervalSinceNow: 0.2];
+	      [loop runUntilDate: next];
+	      app = [NSConnection
+			    rootProxyForConnectionWithRegisteredName: port  
+								host: @""];
+	    }
+	}
+      NS_HANDLER
+	{
+	  return nil;
+	}
+      NS_ENDHANDLER
+    }
+
+  return app;
+}
+
 BOOL
 NSPerformService(NSString *serviceItem, NSPasteboard *pboard)
 {
@@ -1061,12 +1255,8 @@ NSPerformService(NSString *serviceItem, NSPasteboard *pboard)
       msgSel = sel_register_typed_name(name, type);
     }
 
-  NS_DURING
-    {
-      provider = [NSConnection rootProxyForConnectionWithRegisteredName: port  
-								   host: @""];
-    }
-  NS_HANDLER
+  provider = GSContactApplication(appPath, port, finishBy);
+  if (provider == nil)
     {
       NSRunAlertPanel(nil,
 	[NSString stringWithFormat:
@@ -1074,58 +1264,6 @@ NSPerformService(NSString *serviceItem, NSPasteboard *pboard)
 	@"Continue", nil, nil);
       return NO;
     }
-  NS_ENDHANDLER
-
-  if (provider == nil)
-    {
-      if ([[NSWorkspace sharedWorkspace] launchApplication: appPath] == NO)
-	{
-	  NSRunAlertPanel(nil,
-	    [NSString stringWithFormat:
-		@"Failed to launch service provider for '%@'", serviceItem],
-	    @"Continue", nil, nil);
-	  return NO;		/* Unable to launch.	*/
-	}
-
-      NS_DURING
-	{
-	  provider = [NSConnection
-			rootProxyForConnectionWithRegisteredName: port  
-							    host: @""];
-	  while (provider == nil && [finishBy timeIntervalSinceNow] > 1.0)
-	    {
-	      NSRunLoop	*loop = [NSRunLoop currentRunLoop];
-	      NSDate	*next;
-
-	      [NSTimer scheduledTimerWithTimeInterval: 1.0
-					   invocation: nil
-					      repeats: NO];
-	      next = [NSDate dateWithTimeIntervalSinceNow: 5.0];
-	      [loop runUntilDate: next];
-	      provider = [NSConnection
-			    rootProxyForConnectionWithRegisteredName: port  
-								host: @""];
-	    }
-	}
-      NS_HANDLER
-	{
-	  NSRunAlertPanel(nil,
-	    [NSString stringWithFormat:
-		@"Failed to contact service provider for '%@'", serviceItem],
-	    @"Continue", nil, nil);
-	  return NO;
-	}
-      NS_ENDHANDLER
-    }
-
-  if (provider == nil)
-    {
-      NSRunAlertPanel(nil,
-	[NSString stringWithFormat:
-	    @"Failed to contact service provider for '%@'", serviceItem],
-	    @"Continue", nil, nil);
-      return NO;		/* Unable to contact.	*/
-    }
 
   connection = [(NSDistantObject*)provider connectionForProxy];
   seconds = [finishBy timeIntervalSinceNow];
@@ -1135,7 +1273,12 @@ NSPerformService(NSString *serviceItem, NSPasteboard *pboard)
   msgImp = get_imp(fastClass(provider), msgSel);
   NS_DURING
     {
-      (*msgImp)(provider, msgSel, pboard, userData, &error);
+      [provider performService: selName
+		withPasteboard: pboard
+		      userData: userData
+			 error: &error];
+
+//      (*msgImp)(provider, msgSel, pboard, userData, &error);
     }
   NS_HANDLER
     {
