@@ -28,13 +28,16 @@
 
 #include <gnustep/gui/config.h>
 #include <Foundation/NSArray.h>
+#include <Foundation/NSException.h>
 #include <Foundation/NSString.h>
 
 #include <AppKit/NSBox.h>
+#include <AppKit/NSColor.h>
+#include <AppKit/NSGraphics.h>
 #include <AppKit/NSTextFieldCell.h>
 
 @interface NSBox (Private)
-- (NSRect)calcSizes;
+- (NSRect) calcSizesAllowingNegative: (BOOL)aFlag;
 @end
 
 @implementation NSBox
@@ -42,118 +45,117 @@
 //
 // Class methods
 //
-+ (void)initialize
++ (void) initialize
 {
   if (self == [NSBox class])
     {
       // Initial version
-      [self setVersion:1];
+      [self setVersion: 1];
     }
 }
 
 //
 // Instance methods
 //
-- initWithFrame:(NSRect)frameRect
+- (id) initWithFrame: (NSRect)frameRect
 {
-	[super initWithFrame:frameRect];
+  [super initWithFrame: frameRect];
 	
-	cell = [[NSTextFieldCell alloc] initTextCell:@"Title"];
-	[cell setAlignment: NSCenterTextAlignment];
-	[cell setBordered: NO];
-	[cell setEditable: NO];
-	[cell setDrawsBackground: YES];
-	[cell setBackgroundColor: [window backgroundColor]];
-	offsets.width = 5;
-	offsets.height = 5;
-	border_rect = bounds;
-	border_type = NSLineBorder;
-	title_position = NSAtTop;
-	title_rect = NSZeroRect;
-	ASSIGN(content_view, [NSView new]);
-	[super addSubview:content_view];
+  cell = [[NSTextFieldCell alloc] initTextCell: @"Title"];
+  [cell setAlignment: NSCenterTextAlignment];
+  [cell setBordered: NO];
+  [cell setEditable: NO];
+  [cell setDrawsBackground: YES];
+  [cell setBackgroundColor: [window backgroundColor]];
+  offsets.width = 5;
+  offsets.height = 5;
+  border_rect = bounds;
+  border_type = NSGrooveBorder;
+  title_position = NSAtTop;
+  title_rect = NSZeroRect;
+  content_view = [NSView new];
+  [super addSubview: content_view];
+  [content_view release];
 
-	return self;
+  return self;
 }
 
-- (void)dealloc
+- (void) dealloc
 {
-	if (cell) 
-		[cell release];
-	if (content_view) 
-		[content_view release];
+  if (cell) 
+    [cell release];
 
-	[super dealloc];
+  [super dealloc];
 }
 
 //
 // Getting and Modifying the Border and Title 
 //
-- (NSRect)borderRect
+- (NSRect) borderRect
 {
   return border_rect;
 }
 
-- (NSBorderType)borderType
+- (NSBorderType) borderType
 {
   return border_type;
 }
 
-- (void)setBorderType:(NSBorderType)aType
+- (void) setBorderType: (NSBorderType)aType
 {
   if (border_type != aType)
     {
       border_type = aType;
-      [content_view setFrame: [self calcSizes]];
+      [content_view setFrame: [self calcSizesAllowingNegative: NO]];
       [self setNeedsDisplay: YES];
     }
 }
 
-- (void)setTitle:(NSString *)aString
+- (void) setTitle: (NSString *)aString
 {
-  [cell setStringValue:aString];
-  [content_view setFrame: [self calcSizes]];
+  [cell setStringValue: aString];
+  [content_view setFrame: [self calcSizesAllowingNegative: NO]];
   [self setNeedsDisplay: YES];
 }
 
-- (void)setTitleFont:(NSFont *)fontObj
+- (void) setTitleFont: (NSFont *)fontObj
 {
-  [cell setFont:fontObj];
-  [content_view setFrame: [self calcSizes]];
+  [cell setFont: fontObj];
+  [content_view setFrame: [self calcSizesAllowingNegative: NO]];
   [self setNeedsDisplay: YES];
 }
 
-- (void)setTitlePosition:(NSTitlePosition)aPosition
+- (void) setTitlePosition: (NSTitlePosition)aPosition
 {
   if (title_position != aPosition)
     {
       title_position = aPosition;
-      [content_view setFrame: [self calcSizes]];
+      [content_view setFrame: [self calcSizesAllowingNegative: NO]];
       [self setNeedsDisplay: YES];
     }
 }
 
-- (NSString *)title
+- (NSString*) title
 {
   return [cell stringValue];
 }
 
-- (id)titleCell
+- (id) titleCell
 {
   return cell;
 }
 
-- (NSFont *)titleFont
+- (NSFont*) titleFont
 {
   return [cell font];
 }
 
-- (NSTitlePosition)titlePosition
+- (NSTitlePosition) titlePosition
 {
   return title_position;
 }
 
-- (NSRect)titleRect
+- (NSRect) titleRect
 {
   return title_rect;
 }
@@ -161,50 +163,57 @@
 //
 // Setting and Placing the Content View 
 //
-- (id)contentView
+- (id) contentView
 {
   return content_view;
 }
 
-- (NSSize)contentViewMargins
+- (NSSize) contentViewMargins
 {
   return offsets;
 }
 
-- (void)setContentView:(NSView *)aView
+- (void) setContentView: (NSView*)aView
 {
-  [aView retain];
-  [content_view release];
-  content_view = aView;
-
-  [self replaceSubview:content_view with:aView];
-  [content_view setFrame: [self calcSizes]];
+  if (aView)
+    {
+      [super replaceSubview: content_view with: aView];
+      content_view = aView;
+      [content_view setFrame: [self calcSizesAllowingNegative: NO]];
+    }
 }
 
-- (void)setContentViewMargins:(NSSize)offsetSize
+- (void) setContentViewMargins: (NSSize)offsetSize
 {
+  NSAssert(offsetSize.width >= 0 && offsetSize.height >= 0,
+	@"illegal margins supplied");
+
   offsets = offsetSize;
-  [content_view setFrame: [self calcSizes]];
+  [content_view setFrame: [self calcSizesAllowingNegative: NO]];
   [self setNeedsDisplay: YES];
 }
 
 //
 // Resizing the Box 
 //
-- (void)setFrameFromContentFrame:(NSRect)contentFrame
+- (void) setFrameFromContentFrame: (NSRect)contentFrame
 {
   // First calc the sizes to see how much we are off by
-  NSRect r = [self calcSizes];
+  NSRect r = [self calcSizesAllowingNegative: YES];
   NSRect f = [self frame];
 
+  NSAssert(contentFrame.size.width >= 0 && contentFrame.size.height >= 0,
+	@"illegal content frame supplied");
+
   // Add the difference to the frame
   f.size.width = f.size.width + (contentFrame.size.width - r.size.width);
   f.size.height = f.size.height + (contentFrame.size.height - r.size.height);
 
   [self setFrame: f];
+  [content_view setFrame: [self calcSizesAllowingNegative: NO]];  
 }
 
-- (void)sizeToFit
+- (void) sizeToFit
 {
   NSRect r = NSZeroRect;
   id o, e = [[content_view subviews] objectEnumerator];
@@ -229,55 +238,68 @@
 - (void) resizeWithOldSuperviewSize: (NSSize)oldSize
 {
   [super resizeWithOldSuperviewSize: oldSize];
-  [content_view setFrame: [self calcSizes]];
+  [content_view setFrame: [self calcSizesAllowingNegative: NO]];
 }
 
 //
 // Managing the NSView Hierarchy 
 //
-- (void)addSubview:(NSView *)aView
+- (void) addSubview: (NSView*)aView
 {
-  // Subviews get added to our content view's list
-  [content_view addSubview:aView];
+  [content_view addSubview: aView];
+}
+
+- (void) addSubview: (NSView*)aView
+         positioned: (NSWindowOrderingMode)place
+         relativeTo: (NSView*)otherView
+{
+  [content_view addSubview: aView positioned: place relativeTo: otherView];
+}
+
+- (void) replaceSubview: (NSView *)aView with: (NSView*) newView
+{
+  [content_view replaceSubview: aView with: newView];
 }
 
 //
 // Displaying
 //
-- (void)drawRect:(NSRect)rect
+- (void) drawRect: (NSRect)rect
 {
+  rect = NSIntersectionRect(bounds, rect);
   // Fill inside
   [[window backgroundColor] set];
-  NSRectFill(bounds);
+  NSRectFill(rect);
 
   // Draw border
-  switch(border_type)
+  switch (border_type)
     {
-    case NSNoBorder:
+    case NSNoBorder: 
       break;
-    case NSLineBorder:
+    case NSLineBorder: 
+      [[NSColor controlDarkShadowColor] set];
       NSFrameRect(border_rect);
       break;
-    case NSBezelBorder:
-      NSDrawGrayBezel(border_rect, bounds);
+    case NSBezelBorder: 
+      NSDrawGrayBezel(border_rect, rect);
       break;
-    case NSGrooveBorder:
-      NSDrawGroove(border_rect, bounds);
+    case NSGrooveBorder: 
+      NSDrawGroove(border_rect, rect);
       break;
     }
 
   // Draw title
-  switch(title_position)
+  switch (title_position)
     {
-    case NSNoTitle:
+    case NSNoTitle: 
       // Nothing to do
       break;
-    case NSAboveTop:
-    case NSAtTop:
-    case NSBelowTop:
-    case NSAboveBottom:
-    case NSAtBottom:
-    case NSBelowBottom:
+    case NSAboveTop: 
+    case NSAtTop: 
+    case NSBelowTop: 
+    case NSAboveBottom: 
+    case NSAtBottom: 
+    case NSBelowBottom: 
       [cell setBackgroundColor: [window backgroundColor]];
       [cell drawWithFrame: title_rect inView: self];
     }
@@ -286,9 +308,9 @@
 //
 // NSCoding protocol
 //
-- (void)encodeWithCoder:aCoder
+- (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  [super encodeWithCoder:aCoder];
+  [super encodeWithCoder: aCoder];
 
   [aCoder encodeObject: cell];
   [aCoder encodeObject: content_view];
@@ -299,18 +321,21 @@
   [aCoder encodeValueOfObjCType: @encode(NSTitlePosition) at: &title_position];
 }
 
-- initWithCoder:aDecoder
+- (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [super initWithCoder:aDecoder];
+  id aview;
+  [super initWithCoder: aDecoder];
 
-  cell = [aDecoder decodeObject];
-  content_view = [aDecoder decodeObject];
+  [aDecoder decodeValueOfObjCType: @encode(id) at: &cell];
+  aview = [aDecoder decodeObject];
+  [self setContentView: aview];
   offsets = [aDecoder decodeSize];
   border_rect = [aDecoder decodeRect];
   title_rect = [aDecoder decodeRect];
-  [aDecoder decodeValueOfObjCType: @encode(NSBorderType) at: &border_type];
+  [aDecoder decodeValueOfObjCType: @encode(NSBorderType)
+			       at: &border_type];
   [aDecoder decodeValueOfObjCType: @encode(NSTitlePosition) 
-	    at: &title_position];
+			       at: &title_position];
 
   return self;
 }
@@ -319,21 +344,21 @@
 
 @implementation NSBox (Private)
 
-- (NSRect)calcSizes
+- (NSRect) calcSizesAllowingNegative: (BOOL)aFlag
 {
   NSRect r = NSZeroRect;
 
   switch (title_position)
     {
-    case NSNoTitle:
+    case NSNoTitle: 
       {
 	NSSize borderSize = [NSCell sizeForBorderType: border_type];
 	border_rect = bounds;
 	title_rect = NSZeroRect;
 
 	// Add the offsets to the border rect
-	r.origin.x = border_rect.origin.x + offsets.width + borderSize.width;
-	r.origin.y = border_rect.origin.y + offsets.height + borderSize.height;
+	r.origin.x = offsets.width + borderSize.width;
+	r.origin.y = offsets.height + borderSize.height;
 	r.size.width = border_rect.size.width - (2 * offsets.width)
 	  - (2 * borderSize.width);
 	r.size.height = border_rect.size.height - (2 * offsets.height)
@@ -341,7 +366,7 @@
 
 	break;
       }
-    case NSAboveTop:
+    case NSAboveTop: 
       {
 	NSSize titleSize = [cell cellSize];
 	NSSize borderSize = [NSCell sizeForBorderType: border_type];
@@ -373,7 +398,7 @@
 
 	break;
       }
-    case NSBelowTop:
+    case NSBelowTop: 
       {
 	NSSize titleSize = [cell cellSize];
 	NSSize borderSize = [NSCell sizeForBorderType: border_type];
@@ -407,7 +432,7 @@
 
 	break;
       }
-    case NSAtTop:
+    case NSAtTop: 
       {
 	NSSize titleSize = [cell cellSize];
 	NSSize borderSize = [NSCell sizeForBorderType: border_type];
@@ -443,7 +468,7 @@
 
 	break;
       }
-    case NSAtBottom:
+    case NSAtBottom: 
       {
 	NSSize titleSize = [cell cellSize];
 	NSSize borderSize = [NSCell sizeForBorderType: border_type];
@@ -480,7 +505,7 @@
 
 	break;
       }
-    case NSBelowBottom:
+    case NSBelowBottom: 
       {
 	NSSize titleSize = [cell cellSize];
 	NSSize borderSize = [NSCell sizeForBorderType: border_type];
@@ -512,7 +537,7 @@
 
 	break;
       }
-    case NSAboveBottom:
+    case NSAboveBottom: 
       {
 	NSSize titleSize = [cell cellSize];
 	NSSize borderSize = [NSCell sizeForBorderType: border_type];
@@ -547,6 +572,14 @@
       }
     }
 
+  if (!aFlag)
+    {
+      if (r.size.width < 0)
+	r.size.width = 0;
+      if (r.size.height < 0)
+	r.size.height = 0;
+    }
+  
   return r;
 }
 
