@@ -1,4 +1,4 @@
-/* 
+/*
    NSBrowser.m
 
    Control to display and select from hierarchal lists
@@ -9,14 +9,14 @@
    Date: 1996
    Author:  Felipe A. Rodriguez <far@ix.netcom.com>
    Date: August 1998
-   
+
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
@@ -26,7 +26,7 @@
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-*/ 
+*/
 
 #include <gnustep/gui/config.h>
 #include <AppKit/NSBrowser.h>
@@ -38,8 +38,10 @@
 #include <AppKit/NSCell.h>
 #include <AppKit/NSColor.h>
 #include <AppKit/NSScrollView.h>
+#include <AppKit/NSGraphics.h>
 #include <AppKit/NSMatrix.h>
 #include <AppKit/NSTextFieldCell.h>
+#include <AppKit/PSOperators.h>
 
 #define COLUMN_SEP 6
 
@@ -56,15 +58,15 @@
   NSString *_columnTitle;
 }
 
-- (void)setIsLoaded:(BOOL)flag;
+- (void)setIsLoaded: (BOOL)flag;
 - (BOOL)isLoaded;
-- (void)setColumnScrollView:(id)aView;
+- (void)setColumnScrollView: (id)aView;
 - columnScrollView;
-- (void)setColumnMatrix:(id)aMatrix;
+- (void)setColumnMatrix: (id)aMatrix;
 - columnMatrix;
-- (void)setNumberOfRows:(int)num;
+- (void)setNumberOfRows: (int)num;
 - (int)numberOfRows;
-- (void)setColumnTitle:(NSString *)aString;
+- (void)setColumnTitle: (NSString *)aString;
 - (NSString *)columnTitle;
 - emptyView;
 
@@ -72,7 +74,7 @@
 
 @implementation NSBrowserColumn
 
-- init
+- (id) init
 {
   [super init];
 
@@ -82,7 +84,7 @@
   return self;
 }
 
-- (void)dealloc
+- (void) dealloc
 {
   [_columnScrollView release];
   [_columnMatrix release];
@@ -91,21 +93,19 @@
   [super dealloc];
 }
 
-- (void)setIsLoaded:(BOOL)flag
+- (void) setIsLoaded: (BOOL)flag
 {
   _isLoaded = flag;
 }
 
-- (BOOL)isLoaded
+- (BOOL) isLoaded
 {
   return _isLoaded;
 }
 
-- (void)setColumnScrollView:(id)aView
+- (void) setColumnScrollView: (id)aView
 {
-  [aView retain];
-  [_columnScrollView release];
-  _columnScrollView = aView;
+  ASSIGN(_columnScrollView, aView);
 }
 
 - columnScrollView
@@ -113,11 +113,9 @@
   return _columnScrollView;
 }
 
-- (void)setColumnMatrix:(id)aMatrix
+- (void)setColumnMatrix: (id)aMatrix
 {
-  [aMatrix retain];
-  [_columnMatrix release];
-  _columnMatrix = aMatrix;
+  ASSIGN(_columnMatrix, aMatrix);
 }
 
 - columnMatrix
@@ -125,7 +123,7 @@
   return _columnMatrix;
 }
 
-- (void)setNumberOfRows:(int)num
+- (void) setNumberOfRows: (int)num
 {
   _numberOfRows = num;
 }
@@ -135,7 +133,7 @@
   return _numberOfRows;
 }
 
-- (void)setColumnTitle:(NSString *)aString
+- (void)setColumnTitle: (NSString *)aString
 {
   [aString retain];
   [_columnTitle release];
@@ -158,11 +156,11 @@
 // Private NSBrowser methods
 //
 @interface NSBrowser (Private)
-- (void)_adjustMatrixOfColumn:(int)column;
-- (void)_adjustScrollerFrameOfColumn:(int)column force:(BOOL)flag;
-- (void)_adjustScrollerFrames:(BOOL)flag;
-- (void)_performLoadOfColumn:(int)column;
-- (void)_unloadFromColumn:(int)column;
+- (void)_adjustMatrixOfColumn: (int)column;
+- (void)_adjustScrollerFrameOfColumn: (int)column force: (BOOL)flag;
+- (void)_adjustScrollerFrames: (BOOL)flag;
+- (void)_performLoadOfColumn: (int)column;
+- (void)_unloadFromColumn: (int)column;
 @end
 
 //
@@ -178,22 +176,22 @@
   if (self == [NSBrowser class])
     {
       // Initial version
-      [self setVersion:1];
+      [self setVersion: 1];
     }
 }
 
 //
-// Setting Component Classes 
+// Setting Component Classes
 //
 + (Class)cellClass
 {
   return [NSBrowserCell class];
 }
 
-//
-// Instance methods
-//
-- initWithFrame:(NSRect)rect
+/*
+ * Instance methods
+ */
+- (id) initWithFrame: (NSRect)rect
 {
   NSSize bs;
   NSRect scroller_rect;
@@ -231,8 +229,8 @@
   _browserColumns = [[NSMutableArray alloc] init];
   _titleCell = [NSTextFieldCell new];
   [_titleCell setEditable: NO];
-  [_titleCell setTextColor: [NSColor whiteColor]];
-  [_titleCell setBackgroundColor: [NSColor darkGrayColor]];
+  [_titleCell setTextColor: [NSColor windowFrameTextColor]];
+  [_titleCell setBackgroundColor: [NSColor controlShadowColor]];
   //[_titleCell setBordered: YES];
   //[_titleCell setBezeled: YES];
   [_titleCell setAlignment: NSCenterTextAlignment];
@@ -261,32 +259,55 @@
 }
 
 //
-// Setting the Delegate 
+// Target-actions stuff
+//
+- (void) setTarget: (id)t
+{
+  _target = t;
+}
+
+- (id) target
+{
+  return _target;
+}
+
+- (void) setAction: (SEL)s
+{
+  _action = s;
+}
+
+- (SEL) action
+{
+  return _action;
+}
+
+//
+// Setting the Delegate
 //
 - (id)delegate
 {
   return _browserDelegate;
 }
 
-- (void)setDelegate:(id)anObject
+- (void)setDelegate: (id)anObject
 {
   BOOL flag = NO;
   BOOL both = NO;
 
-  if (![anObject respondsToSelector:
+  if (![anObject respondsToSelector: 
 		  @selector(browser:willDisplayCell:atRow:column:)])
     [NSException raise: NSBrowserIllegalDelegateException
 		 format: @"Delegate does not respond to %s\n",
-		 "browser:willDisplayCell:atRow:column:"];
+		 "browser: willDisplayCell: atRow: column: "];
 
-  if ([anObject respondsToSelector:
+  if ([anObject respondsToSelector: 
 		  @selector(browser:numberOfRowsInColumn:)])
     {
       _passiveDelegate = YES;
       flag = YES;
     }
 
-  if ([anObject respondsToSelector:
+  if ([anObject respondsToSelector: 
 		  @selector(browser:createRowsForColumn:inMatrix:)])
     {
       _passiveDelegate = NO;
@@ -302,14 +323,14 @@
   if (!flag)
     [NSException raise: NSBrowserIllegalDelegateException
 		 format: @"Delegate does not respond to %s or %s\n",
-		 "browser:numberOfRowsInColumn:",
-		 "browser:createRowsForColumn:inMatrix:"];
+		 "browser: numberOfRowsInColumn: ",
+		 "browser: createRowsForColumn: inMatrix: "];
 
   if (both)
     [NSException raise: NSBrowserIllegalDelegateException
 		 format: @"Delegate responds to both %s and %s\n",
-		 "browser:numberOfRowsInColumn:",
-		 "browser:createRowsForColumn:inMatrix:"];
+		 "browser: numberOfRowsInColumn: ",
+		 "browser: createRowsForColumn: inMatrix: "];
 
   [anObject retain];
   [_browserDelegate release];
@@ -317,7 +338,7 @@
 }
 
 //
-// Target and Action 
+// Target and Action
 //
 - (SEL)doubleAction
 {
@@ -329,13 +350,13 @@
   return [self sendAction: [self action] to: [self target]];
 }
 
-- (void)setDoubleAction:(SEL)aSelector
+- (void)setDoubleAction: (SEL)aSelector
 {
   _doubleAction = aSelector;
 }
 
 //
-// Setting Component Classes 
+// Setting Component Classes
 //
 - (id)cellPrototype
 {
@@ -347,7 +368,7 @@
   return _browserMatrixClass;
 }
 
-- (void)setCellClass:(Class)classId
+- (void)setCellClass: (Class)classId
 {
   _browserCellClass = classId;
 
@@ -355,32 +376,32 @@
   [self setCellPrototype: [[[_browserCellClass alloc] init] autorelease]];
 }
 
-- (void)setCellPrototype:(NSCell *)aCell
+- (void)setCellPrototype: (NSCell *)aCell
 {
   [aCell retain];
   [_browserCellPrototype release];
   _browserCellPrototype = aCell;
 }
 
-- (void)setMatrixClass:(Class)classId
+- (void)setMatrixClass: (Class)classId
 {
   _browserMatrixClass = classId;
 }
 
 //
-// Setting NSBrowser Behavior 
+// Setting NSBrowser Behavior
 //
 - (BOOL)reusesColumns
 {
   return _reusesColumns;
 }
 
-- (void)setReusesColumns:(BOOL)flag
+- (void)setReusesColumns: (BOOL)flag
 {
   _reusesColumns = flag;
 }
 
-- (void)setTakesTitleFromPreviousColumn:(BOOL)flag
+- (void)setTakesTitleFromPreviousColumn: (BOOL)flag
 {
   _takesTitleFromPreviousColumn = flag;
 }
@@ -391,7 +412,7 @@
 }
 
 //
-// Allowing Different Types of Selection 
+// Allowing Different Types of Selection
 //
 - (BOOL)allowsBranchSelection
 {
@@ -408,17 +429,17 @@
   return _allowsMultipleSelection;
 }
 
-- (void)setAllowsBranchSelection:(BOOL)flag
+- (void)setAllowsBranchSelection: (BOOL)flag
 {
   _allowsBranchSelection = flag;
 }
 
-- (void)setAllowsEmptySelection:(BOOL)flag
+- (void)setAllowsEmptySelection: (BOOL)flag
 {
   _allowsEmptySelection = flag;
 }
 
-- (void)setAllowsMultipleSelection:(BOOL)flag
+- (void)setAllowsMultipleSelection: (BOOL)flag
 {
   _allowsMultipleSelection = flag;
 }
@@ -436,20 +457,20 @@
   return _sendsActionOnArrowKeys;
 }
 
-- (void)setAcceptsArrowKeys:(BOOL)flag
+- (void)setAcceptsArrowKeys: (BOOL)flag
 {
   _acceptsArrowKeys = flag;
 }
 
-- (void)setSendsActionOnArrowKeys:(BOOL)flag
+- (void)setSendsActionOnArrowKeys: (BOOL)flag
 {
   _sendsActionOnArrowKeys = flag;
 }
 
 //
-// Showing a Horizontal Scroller 
+// Showing a Horizontal Scroller
 //
-- (void)setHasHorizontalScroller:(BOOL)flag
+- (void)setHasHorizontalScroller: (BOOL)flag
 {
   _hasHorizontalScroller = flag;
 
@@ -465,7 +486,7 @@
 }
 
 //
-// Setting the NSBrowser's Appearance 
+// Setting the NSBrowser's Appearance
 //
 - (int)maxVisibleColumns
 {
@@ -482,7 +503,7 @@
   return _separatesColumns;
 }
 
-- (void)setMaxVisibleColumns:(int)columnCount
+- (void)setMaxVisibleColumns: (int)columnCount
 {
   int i, count = [_browserColumns count];
 
@@ -498,7 +519,7 @@
   [self _adjustScrollerFrames: NO];
 }
 
-- (void)setMinColumnWidth:(int)columnWidth
+- (void)setMinColumnWidth: (int)columnWidth
 {
   float sw = [NSScroller scrollerWidth];
   NSSize bs = [NSCell sizeForBorderType: NSBezelBorder];
@@ -517,7 +538,7 @@
   [self tile];
 }
 
-- (void)setSeparatesColumns:(BOOL)flag
+- (void)setSeparatesColumns: (BOOL)flag
 {
   _separatesColumns = flag;
 
@@ -525,7 +546,7 @@
 }
 
 //
-// Manipulating Columns 
+// Manipulating Columns
 //
 - (void)addColumn
 {
@@ -547,7 +568,7 @@
   [_browserColumns addObject: bc];
 }
 
-- (int)columnOfMatrix:(NSMatrix *)matrix
+- (int)columnOfMatrix: (NSMatrix *)matrix
 {
   NSBrowserColumn *bc;
   int i, count = [_browserColumns count];
@@ -564,7 +585,7 @@
   return NSNotFound;
 }
 
-- (void)displayAllColumns
+- (void) displayAllColumns
 {
   int i, count = [_browserColumns count];
 
@@ -578,7 +599,7 @@
     }
 }
 
-- (void)displayColumn:(int)column
+- (void) displayColumn: (int)column
 {
   NSBrowserColumn *bc;
 
@@ -589,7 +610,7 @@
   bc = [_browserColumns objectAtIndex: column];
 
   // Ask the delegate for the column title
-  if ([_browserDelegate respondsToSelector:
+  if ([_browserDelegate respondsToSelector: 
 			  @selector(browser:titleOfColumn:)])
     [self setTitle: [_browserDelegate browser: self
 				      titleOfColumn: column]
@@ -695,7 +716,7 @@
     return i + 1;
 }
 
-- (void)reloadColumn:(int)column
+- (void)reloadColumn: (int)column
 {
   NSBrowserColumn *bc;
 
@@ -719,7 +740,7 @@
   [self setLastColumn: column];
 }
 
-- (void)selectAll:(id)sender
+- (void)selectAll: (id)sender
 {
   id matrix = [self matrixInColumn: _lastVisibleColumn];
   [matrix selectAll: sender];
@@ -749,7 +770,12 @@
     return NSNotFound;
 }
 
-- (void)setLastColumn:(int)column
+- (int) selectedRowInColumn: (int)column
+{
+  return [[self matrixInColumn: column] selectedRow];
+}
+
+- (void)setLastColumn: (int)column
 {
   _lastColumnLoaded = column;
 }
@@ -759,7 +785,7 @@
   int i;
 
   // xxx Should we trigger an exception?
-  if (![_browserDelegate respondsToSelector:
+  if (![_browserDelegate respondsToSelector: 
 			   @selector(browser:isColumnValid:)])
     return;
 
@@ -775,11 +801,11 @@
 }
 
 //
-// Manipulating Column Titles 
+// Manipulating Column Titles
 //
-- (void)drawTitle:(NSString *)title
-	   inRect:(NSRect)aRect
-	 ofColumn:(int)column
+- (void)drawTitle: (NSString *)title
+	   inRect: (NSRect)aRect
+	 ofColumn: (int)column
 {
   // Not titled then nothing to draw
   if (![self isTitled])
@@ -798,15 +824,15 @@
   return _isTitled;
 }
 
-- (void)setTitled:(BOOL)flag
+- (void)setTitled: (BOOL)flag
 {
   _isTitled = flag;
 
   [self tile];
 }
 
-- (void)setTitle:(NSString *)aString
-	ofColumn:(int)column
+- (void)setTitle: (NSString *)aString
+	ofColumn: (int)column
 {
   NSBrowserColumn *bc = [_browserColumns objectAtIndex: column];
   [bc setColumnTitle: aString];
@@ -818,7 +844,7 @@
     [self setNeedsDisplayInRect: [self titleFrameOfColumn: column]];
 }
 
-- (NSRect)titleFrameOfColumn:(int)column
+- (NSRect)titleFrameOfColumn: (int)column
 {
   NSRect r;
   int n;
@@ -847,16 +873,16 @@
   return 24;
 }
 
-- (NSString *)titleOfColumn:(int)column
+- (NSString *)titleOfColumn: (int)column
 {
   NSBrowserColumn *bc = [_browserColumns objectAtIndex: column];
   return [bc columnTitle];
 }
 
 //
-// Scrolling an NSBrowser 
+// Scrolling an NSBrowser
 //
-- (void)scrollColumnsLeftBy:(int)shiftAmount
+- (void)scrollColumnsLeftBy: (int)shiftAmount
 {
   // Cannot shift past the zero column
   if ((_firstVisibleColumn - shiftAmount) < 0)
@@ -885,7 +911,7 @@
     [_browserDelegate browserDidScroll: self];
 }
 
-- (void)scrollColumnsRightBy:(int)shiftAmount
+- (void)scrollColumnsRightBy: (int)shiftAmount
 {
   // Cannot shift past the last loaded column
   if ((shiftAmount + _lastVisibleColumn) > _lastColumnLoaded)
@@ -914,7 +940,7 @@
     [_browserDelegate browserDidScroll: self];
 }
 
-- (void)scrollColumnToVisible:(int)column
+- (void)scrollColumnToVisible: (int)column
 {
   int i;
 
@@ -934,7 +960,7 @@
     [self scrollColumnsRightBy: (-i)];
 }
 
-- (void)scrollViaScroller:(NSScroller *)sender
+- (void)scrollViaScroller: (NSScroller *)sender
 {
   NSScrollerPart hit = [sender hitPart];
 
@@ -979,7 +1005,7 @@
     }
 }
 
-- (void)updateScroller
+- (void) updateScroller
 {
   // If there are not enough columns to scroll with
   // then the column must be visible
@@ -995,15 +1021,13 @@
       [_horizontalScroller setEnabled: YES];
     }
 
-	[self display];			// should probably be a setNeedsDisplay FIX ME?
-
-	return;
+  [self setNeedsDisplay: YES];
 }
 
 //
-// Event Handling 
+// Event Handling
 //
-- (void)doClick:(id)sender
+- (void)doClick: (id)sender
 {
   int column = [self columnOfMatrix: sender];
   NSArray *a;
@@ -1014,7 +1038,7 @@
     return;
 
   // Ask delegate if selection is ok
-  if ([_browserDelegate respondsToSelector:
+  if ([_browserDelegate respondsToSelector: 
 				@selector(browser:selectRow:inColumn:)])
     {
       int row = [sender selectedRow];
@@ -1024,12 +1048,12 @@
   else
     {
       // Try the other method
-      if ([_browserDelegate respondsToSelector:
+      if ([_browserDelegate respondsToSelector: 
 			    @selector(browser:selectCellWithString:inColumn:)])
 	{
 	  id c = [sender selectedCell];
 	  shouldSelect = [_browserDelegate browser: self
-					   selectCellWithString:
+					   selectCellWithString: 
 					     [c stringValue]
 					   inColumn: column];
 	}
@@ -1087,11 +1111,10 @@
 
   // Send the action to target
   [self sendAction];
-
-	[self display];			// should probably be a setNeedsDisplay FIX ME?
+  [self setNeedsDisplay: YES];
 }
 
-- (void)doDoubleClick:(id)sender
+- (void)doDoubleClick: (id)sender
 {
   // We have already handled the single click
   // so send the double action
@@ -1099,10 +1122,10 @@
 }
 
 //
-// Getting Matrices and Cells 
+// Getting Matrices and Cells
 //
-- (id)loadedCellAtRow:(int)row
-	       column:(int)column
+- (id)loadedCellAtRow: (int)row
+	       column: (int)column
 {
   NSBrowserColumn *bc;
   NSArray *columnCells;
@@ -1139,7 +1162,7 @@
   return aCell;
 }
 
-- (NSMatrix *)matrixInColumn:(int)column
+- (NSMatrix *)matrixInColumn: (int)column
 {
   NSBrowserColumn *bc = [_browserColumns objectAtIndex: column];
   return [bc columnMatrix];
@@ -1158,7 +1181,7 @@
   return [matrix selectedCell];
 }
 
-- (id)selectedCellInColumn:(int)column
+- (id)selectedCellInColumn: (int)column
 {
   id matrix = [self matrixInColumn: column];
   return [matrix selectedCell];
@@ -1178,9 +1201,9 @@
 }
 
 //
-// Getting Column Frames 
+// Getting Column Frames
 //
-- (NSRect)frameOfColumn:(int)column
+- (NSRect)frameOfColumn: (int)column
 {
   NSRect r = NSZeroRect;
   int n;
@@ -1202,14 +1225,14 @@
   return r;
 }
 
-- (NSRect)frameOfInsideOfColumn:(int)column
+- (NSRect)frameOfInsideOfColumn: (int)column
 {
   // xxx what does this one do?
   return [self frameOfColumn: column];
 }
 
 //
-// Manipulating Paths 
+// Manipulating Paths
 //
 - (NSString *)path
 {
@@ -1221,33 +1244,115 @@
   return _pathSeparator;
 }
 
-- (NSString *)pathToColumn:(int)column
+- (NSString *)pathToColumn: (int)column
 {
-  int i;
-  NSMutableString *s = [NSMutableString stringWithCString: ""];
+  NSMutableString	*s = [_pathSeparator mutableCopy];
+  unsigned		i;
 
-  // Cannot go past the number of loaded columns
+  /*
+   * Cannot go past the number of loaded columns
+   */
   if (column > _lastColumnLoaded)
     column = _lastColumnLoaded + 1;
 
-  [s appendString: _pathSeparator];
   for (i = 0;i < column; ++i)
     {
-      id c = [self selectedCellInColumn: i];
+      id	c = [self selectedCellInColumn: i];
+
       if (i != 0)
 	[s appendString: _pathSeparator];
       [s appendString: [c stringValue]];
     }
-
-  return [[[NSString alloc] initWithString: s] autorelease];
+  /*
+   * We actually return a mutable string, but that's ok since a mutable
+   * string is a string and the documentation specifically says that
+   * people should not depend on methods that return strings to return
+   * immutable strings.
+   */
+  return s;
 }
 
-- (BOOL)setPath:(NSString *)path
+- (BOOL) setPath: (NSString *)path
 {
-  return NO;
+  NSArray	*subStrings;
+  NSString	*aStr;
+  unsigned	numberOfSubStrings;
+  unsigned	i;
+
+  subStrings = [path componentsSeparatedByString: _pathSeparator];
+  [self setLastColumn: 0];
+  numberOfSubStrings = [subStrings count];
+
+  if (numberOfSubStrings == 2)
+    {
+      /* select root path */
+      if ([[subStrings objectAtIndex: 1] length] == 0)
+	{
+	  [self scrollColumnsLeftBy: [_browserColumns count]];
+	  return YES;
+	}
+    }
+
+  // cycle thru str's array created from path
+  for (i = 1; i < numberOfSubStrings; i++)
+    {
+      NSBrowserColumn	*bc = [_browserColumns objectAtIndex: i-1];
+      NSMatrix		*matrix = [bc columnMatrix];
+      NSArray		*cells = [matrix cells];
+      unsigned		j, numOfRows = [cells count];
+      NSBrowserCell	*selectedCell = nil;
+
+      aStr = [subStrings objectAtIndex: i];
+      // find the cell in the browser matrix with the equal to aStr
+      for (j = 0; j < numOfRows; j++)
+	{
+	  NSArray	*cellRow = [cells objectAtIndex: j];
+	  unsigned	 numOfCols, k;
+
+	  numOfCols = [cellRow count];
+	  for (k = 0; k < numOfCols; k++)
+	    {
+	      NSString	*cellString;
+
+	      selectedCell = [cellRow objectAtIndex: k];
+	      cellString = [selectedCell stringValue];
+
+	      if ([cellString isEqualToString: aStr])
+		{
+		  int r, c;
+
+		  k = numOfCols;
+		  j = numOfRows;
+		  if ([matrix getRow: &r column: &c ofCell: selectedCell])
+		    [matrix selectCellAtRow: r column: c];
+		  else
+		    selectedCell = nil;
+		}
+	    }
+	}
+      // if unable to find a cell whose title matches aStr return NO
+      if (!selectedCell)
+	{
+	  NSLog(@"NSBrowser: unable to find cell in matrix\n");
+	  return NO;
+	}
+      // if the cell is not a leaf add a column to the browser for it
+      if (![selectedCell isLeaf])
+	{
+	  [self addColumn];
+	  [self _performLoadOfColumn: i];
+	  [self setLastColumn: i];
+	  [self _adjustMatrixOfColumn: i];
+	  [self scrollColumnsRightBy: 1];
+	}
+      else
+	break;
+    }
+
+  return YES;
 }
 
-- (void)setPathSeparator:(NSString *)aString
+- (void)setPathSeparator: (NSString *)aString
 {
   [aString retain];
   [_pathSeparator release];
@@ -1255,7 +1360,7 @@
 }
 
 //
-// Arranging an NSBrowser's Components 
+// Arranging an NSBrowser's Components
 //
 - (void)tile
 {
@@ -1294,10 +1399,11 @@
 
 }
 
-- (void)drawRect:(NSRect)rect
+- (void)drawRect: (NSRect)rect
 {
   int i;
 
+  NSRectClip(rect);
   // Load the first column if not already done
   if (!_isLoaded)
     {
@@ -1317,39 +1423,39 @@
 }
 
 //
-// Displaying the Control and Cell 
+// Displaying the Control and Cell
 //
-- (void)drawCell:(NSCell *)aCell
+- (void)drawCell: (NSCell *)aCell
 {
 }
 
-- (void)drawCellInside:(NSCell *)aCell
+- (void)drawCellInside: (NSCell *)aCell
 {
 }
 
-- (void)selectCell:(NSCell *)aCell
+- (void)selectCell: (NSCell *)aCell
 {
 }
 
-- (void)updateCell:(NSCell *)aCell
+- (void)updateCell: (NSCell *)aCell
 {
 }
 
-- (void)updateCellInside:(NSCell *)aCell
+- (void)updateCellInside: (NSCell *)aCell
 {
 }
 
 //
 // NSCoding protocol
 //
-- (void)encodeWithCoder:aCoder
+- (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  [super encodeWithCoder:aCoder];
+  [super encodeWithCoder: aCoder];
 }
 
-- initWithCoder:aDecoder
+- (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [super initWithCoder:aDecoder];
+  [super initWithCoder: aDecoder];
 
   return self;
 }
@@ -1361,40 +1467,43 @@
 //
 @implementation NSBrowser (Private)
 
-- (void)_adjustMatrixOfColumn:(int)column
+- (void)_adjustMatrixOfColumn: (int)column
 {
-NSBrowserColumn *bc;
-NSScrollView *sc;
-id matrix;
-NSSize cs, ms;
-NSRect mr;
+  NSBrowserColumn	*bc;
+  NSScrollView		*sc;
+  id			matrix;
+  NSSize		cs, ms;
+  NSRect		mr;
 
-	if (column >= (int)[_browserColumns count])
-		return;
-	
-	bc = [_browserColumns objectAtIndex: column];
-	sc = [bc columnScrollView];
-	matrix = [bc columnMatrix];
-  										// Adjust matrix to fit in scrollview
-	if (sc && matrix && [bc isLoaded])	// do it only if column has been loaded	
-		{								
-		cs = [sc contentSize];
-		ms = [matrix cellSize];
-		ms.width = cs.width;
-		[matrix setCellSize: ms];
-		mr = [matrix frame];
-	
-		if (mr.size.height < cs.height)					// matrix smaller than
-			{											// scrollview's content
-			mr.origin.y = cs.height;					// view requires origin
-			[matrix setFrame: mr];						// adjustment for it to 
-			}											// appear at top
+  if (column >= (int)[_browserColumns count])
+    return;
 
-		[sc setDocumentView: matrix];
-		}
+  bc = [_browserColumns objectAtIndex: column];
+  sc = [bc columnScrollView];
+  matrix = [bc columnMatrix];
+
+  // Adjust matrix to fit in scrollview if column has been loaded
+  if (sc && matrix && [bc isLoaded])
+    {
+      cs = [sc contentSize];
+      ms = [matrix cellSize];
+      ms.width = cs.width;
+      [matrix setCellSize: ms];
+      mr = [matrix frame];
+
+      // matrix smaller than scrollview's content
+      if (mr.size.height < cs.height)
+	{
+	  // view requires origin adjustment for it to appear at top
+	  mr.origin.y = cs.height;
+	  [matrix setFrame: mr];
+	}
+
+      [sc setDocumentView: matrix];
+    }
 }
 
-- (void)_adjustScrollerFrameOfColumn:(int)column force:(BOOL)flag
+- (void)_adjustScrollerFrameOfColumn: (int)column force: (BOOL)flag
 {
   // Only if we've loaded the first column
   if ((_isLoaded) || (flag))
@@ -1416,7 +1525,7 @@ NSRect mr;
     }
 }
 
-- (void)_adjustScrollerFrames:(BOOL)force
+- (void)_adjustScrollerFrames: (BOOL)force
 {
   int i, count = [_browserColumns count];
 
@@ -1439,23 +1548,22 @@ NSRect mr;
 	}
       else
 	{
-	  // If it is not visible
-	  NSBrowserColumn *bc = [_browserColumns objectAtIndex: i];
-	  id sc = [bc columnScrollView];
+	  NSBrowserColumn	*bc = [_browserColumns objectAtIndex: i];
+	  id			sc = [bc columnScrollView];
 
 	  // Remove it from its superview if it has one
 	  if (sc)
-	    if ([sc superview])
-	      {
-		NSLog(@"before: %d\n", [sc retainCount]);
-		[sc removeFromSuperview];
-		NSLog(@"after: %d\n", [sc retainCount]);
-	      }
+	    {
+	      if ([sc superview])
+		{
+		  [sc removeFromSuperview];
+		}
+	    }
 	}
     }
 }
 
-- (void)_performLoadOfColumn:(int)column
+- (void)_performLoadOfColumn: (int)column
 {
   NSBrowserColumn *bc = [_browserColumns objectAtIndex: column];
 
@@ -1472,11 +1580,11 @@ NSRect mr;
 	{
 	  id matrix;
 	  id sc = [bc columnScrollView];
-	  id oldm = [bc columnMatrix];
 	  NSRect matrixRect = {{0, 0}, {100, 100}};
 	  int i;
 
-	  matrix = [[[_browserMatrixClass alloc]		// create a new col matrix
+	  // create a new col matrix
+	  matrix = [[[_browserMatrixClass alloc]
 					initWithFrame: matrixRect
 					mode: NSListModeMatrix
 					prototype: _browserCellPrototype
@@ -1490,7 +1598,8 @@ NSRect mr;
 	  [matrix setAction: @selector(doClick:)];
 	  [matrix setDoubleAction: @selector(doDoubleClick:)];
 
-	  [bc setColumnMatrix: matrix];		// set new col matrix and release old
+	  // set new col matrix and release old
+	  [bc setColumnMatrix: matrix];
 	  [sc setDocumentView: matrix];
 
 	  // Now loop through the cells and load each one
@@ -1507,10 +1616,10 @@ NSRect mr;
 	{
 	  id matrix;
 	  id sc = [bc columnScrollView];
-	  id oldm = [bc columnMatrix];
 	  NSRect matrixRect = {{0, 0}, {100, 100}};
 
-	  matrix = [[[_browserMatrixClass alloc]		// create a new col matrix
+	  // create a new col matrix
+	  matrix = [[[_browserMatrixClass alloc]
 					initWithFrame: matrixRect
 					mode: NSListModeMatrix
 					prototype: _browserCellPrototype
@@ -1524,19 +1633,22 @@ NSRect mr;
 	  [matrix setAction: @selector(doClick:)];
 	  [matrix setDoubleAction: @selector(doDoubleClick:)];
 
-	  [bc setColumnMatrix: matrix];		// set new col matrix and release old
+	  // set new col matrix and release old
+	  [bc setColumnMatrix: matrix];
 	  [sc setDocumentView: matrix];
 
 	  // Tell the delegate to create the rows
-	  [_browserDelegate browser: self createRowsForColumn: column
-			    inMatrix: matrix];
+	  [_browserDelegate browser: self
+		createRowsForColumn: column
+			   inMatrix: matrix];
 	}
     }
 
   [bc setIsLoaded: YES];
+  [self setNeedsDisplayInRect: [self frameOfColumn: column]];
 }
 
-- (void)_unloadFromColumn:(int)column
+- (void)_unloadFromColumn: (int)column
 {
   int i, count = [_browserColumns count];
 
@@ -1556,12 +1668,11 @@ NSRect mr;
     }
 }
 
-- (void)setFrame:(NSRect)frameRect
+- (void) resizeSubviewsWithOldSize: (NSSize)oldSize
 {
-NSLog (@"NSBrowser setFrame ");
-  	[super setFrame:frameRect];
-	[self tile];								// recalc browser's elements
-	[self _adjustScrollerFrames:YES];			// adjust browser's matrix
+  [super resizeSubviewsWithOldSize: oldSize];
+  [self tile];
+  [self _adjustScrollerFrames: YES];
 }
 
 @end
