@@ -31,11 +31,9 @@
 #include <gnustep/gui/config.h>
 #include <AppKit/NSColorWell.h>
 #include <AppKit/NSColor.h>
+#include <AppKit/NSGraphics.h>
 
 
-#define XRBW 1.0								// half the width of the bevel
-#define XRHW 5.0								// width of border/handle
-
 @implementation NSColorWell
 
 //
@@ -43,14 +41,14 @@
 //
 + (void)initialize
 {
-	if (self == [NSColorWell class])
-		[self setVersion:1];								// Initial version
+  if (self == [NSColorWell class])
+    [self setVersion: 1];
 }
 
 //
 // Instance methods
 //
-- initWithFrame:(NSRect)frameRect
+- (id) initWithFrame: (NSRect)frameRect
 {
   [super initWithFrame: frameRect];
 
@@ -67,33 +65,55 @@
   [super dealloc];
 }
 
-//
-// Drawing
-//
-- (void)drawRect:(NSRect)rect
+/*
+ * Drawing
+ */
+- (void) drawRect: (NSRect)rect
 {
-NSRect inside = rect;
+  NSRect aRect = bounds;
   
-	if (is_bordered)										// if well has a 
-		[self drawBorderRect: rect];						// border draw it
+  if (NSIntersectsRect(aRect, rect) == NO)
+    return;
 
-	inside.origin.x += XRBW + XRHW + XRBW + XRBW;			// calc interior
-	inside.origin.y += XRBW + XRHW + XRBW;					// rect
-	inside.size.width -= (4*XRBW + XRHW + XRHW) + XRBW;
-	inside.size.height -= (4*XRBW + XRHW + XRHW) + XRBW;
-	[self drawWellInside: inside];							// draw interior
+  if (is_bordered)
+    {
+      /*
+       * Draw outer frame.
+       */
+      NSDrawButton(aRect, rect);
+
+      /*
+       * Fill in control color.
+       */
+      aRect = NSInsetRect(aRect, 2.0, 2.0);
+      [[NSColor controlColor] set];
+      NSRectFill(NSIntersectionRect(aRect, rect));
+
+      /*
+       * Draw inner frame.
+       */
+      aRect = NSInsetRect(aRect, 5.0, 5.0);
+      NSDrawGrayBezel(aRect, rect);
+      
+      aRect = NSInsetRect(aRect, 2.0, 2.0);
+    }
+  else
+    aRect = NSInsetRect(aRect, 9.0, 9.0);
+
+  [self drawWellInside: NSIntersectionRect(aRect, rect)];
 }
 
-- (void)drawWellInside:(NSRect)insideRect
+- (void) drawWellInside: (NSRect)insideRect
 {
-	[the_color set];
-	NSRectFill(insideRect);									// fill interior	
+  if (NSIsEmptyRect(insideRect))
+    return;
+  [the_color drawSwatchInRect: insideRect];
 }
 
 //
 // Activating 
 //
-- (void)activate:(BOOL)exclusive
+- (void)activate: (BOOL)exclusive
 {
   is_active = YES;
 }
@@ -116,14 +136,14 @@ NSRect inside = rect;
   return the_color;
 }
 
-- (void)setColor:(NSColor *)color
+- (void)setColor: (NSColor *)color
 {
   ASSIGN(the_color, color);
 }
 
-- (void)takeColorFrom:(id)sender
+- (void)takeColorFrom: (id)sender
 {
-  if ([sender respondsToSelector:@selector(color)])
+  if ([sender respondsToSelector: @selector(color)])
     ASSIGN(the_color, [sender color]);
 }
 
@@ -135,7 +155,7 @@ NSRect inside = rect;
   return is_bordered;
 }
 
-- (void)setBordered:(BOOL)bordered
+- (void)setBordered: (BOOL)bordered
 {
   is_bordered = bordered;
   [self display];
@@ -144,46 +164,23 @@ NSRect inside = rect;
 //
 // NSCoding protocol
 //
-- (void)encodeWithCoder:aCoder
+- (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  [super encodeWithCoder:aCoder];
+  [super encodeWithCoder: aCoder];
   [aCoder encodeObject: the_color];
-  [aCoder encodeValueOfObjCType:@encode(BOOL) at: &is_active];
-  [aCoder encodeValueOfObjCType:@encode(BOOL) at: &is_bordered];
+  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &is_active];
+  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &is_bordered];
 }
 
-- initWithCoder:aDecoder
+- (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [super initWithCoder:aDecoder];
-  the_color = [aDecoder decodeObject];
-  [aDecoder decodeValueOfObjCType:@encode(BOOL) at: &is_active];
-  [aDecoder decodeValueOfObjCType:@encode(BOOL) at: &is_bordered];
+  [super initWithCoder: aDecoder];
+  [aDecoder decodeValueOfObjCType: @encode(id) at: &the_color];
+  [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &is_active];
+  [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &is_bordered];
 
   return self;
 }
 
 @end
 
-//
-// GNUstep backend methods
-//
-@implementation NSColorWell (GNUstepBackend)
-
-- (void)drawBorderRect:(NSRect)aRect
-{
-NSRect inside;
-
-	[[NSColor lightGrayColor] set];
-	NSRectFill(bounds);									// fill the area with
-														// gray first
-	NSDrawButton(aRect, aRect);							// draw outer border
-
-	inside = aRect;										// calc inner border
-	inside.origin.x += XRBW + XRHW;						// rect
-	inside.origin.y += XRBW + XRHW;
-	inside.size.width -= (XRBW + XRBW + XRHW + XRHW);
-	inside.size.height -= (XRBW + XRBW + XRHW + XRHW);
-	NSDrawGrayBezel(inside, inside);					// draw inner border
-}
-
-@end
