@@ -47,7 +47,7 @@
 #include <AppKit/IMLoading.h>
 #include <AppKit/GMAppKit.h>
 
-#include <extensions/GMArchiver.h>
+#include <AppKit/GMArchiver.h>
 
 
 
@@ -98,7 +98,20 @@
   return self;
 }
 
+- (void) initDefaults
+{
+  [super initDefaults];
+  menu_exclude = YES;		// Don't show panels in windows menu.
+}
+
 - (BOOL) canBecomeKeyWindow
+{
+  if (_becomesKeyOnlyIfNeeded)
+    return NO;
+  return YES;
+}
+
+- (BOOL) canBecomeMainWindow
 {
   return NO;
 }
@@ -153,12 +166,12 @@
 //
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  [super encodeWithCoder:aCoder];
+  [super encodeWithCoder: aCoder];
 }
 
-- initWithCoder: (NSCoder*)aDecoder
+- (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [super initWithCoder:aDecoder];
+  [super initWithCoder: aDecoder];
 
   return self;
 }
@@ -378,7 +391,7 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 		NSViewWidthSizable | NSViewHeightSizable | NSViewMaxYMargin];
       [messageField setEditable: NO];
       [messageField setSelectable: NO];
-      [messageField setBordered: NO];
+      [messageField setBezeled: NO];
       [messageField setDrawsBackground: NO];
       [messageField setAlignment: NSCenterTextAlignment];
       [messageField setStringValue: @""];
@@ -392,7 +405,7 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
       [titleField setAutoresizingMask: NSViewWidthSizable | NSViewMinYMargin];
       [titleField setEditable: NO];
       [titleField setSelectable: NO];
-      [titleField setBordered: NO];
+      [titleField setBezeled: NO];
       [titleField setDrawsBackground: NO];
       [titleField setStringValue: @""];
       [titleField setFont: [NSFont systemFontOfSize: 18.0]];
@@ -678,6 +691,10 @@ NSRunAlertPanel(NSString *title,
       standardAlertPanel = panel;
     }
 
+  [panel center];
+  if ([NSApp isActive])
+    [panel makeKeyWindow];
+  [panel orderFrontRegardless];
   result = [panel runModal];
   NSReleaseAlertPanel(panel);
   return result;
@@ -699,6 +716,10 @@ NSRunCriticalAlertPanel(NSString *title,
 		defaultButton, alternateButton, otherButton, ap);
   va_end (ap);
 
+  [panel center];
+  if ([NSApp isActive])
+    [panel makeKeyWindow];
+  [panel orderFrontRegardless];
   result = [panel runModal];
   NSReleaseAlertPanel(panel);
   return result;
@@ -720,6 +741,10 @@ NSRunInformationalAlertPanel(NSString *title,
 		defaultButton, alternateButton, otherButton, ap);
   va_end (ap);
 
+  [panel center];
+  if ([NSApp isActive])
+    [panel makeKeyWindow];
+  [panel orderFrontRegardless];
   result = [panel runModal];
   NSReleaseAlertPanel(panel);
   return result;
@@ -782,6 +807,10 @@ NSRunLocalizedAlertPanel(NSString *table,
       standardAlertPanel = panel;
     }
 
+  [panel center];
+  if ([NSApp isActive])
+    [panel makeKeyWindow];
+  [panel orderFrontRegardless];
   result = [panel runModal];
   NSReleaseAlertPanel(panel);
   return result;
