@@ -27,7 +27,13 @@
 */ 
 
 #include "config.h"
-#include <gnustep/xdps/PXKCachedImageRep.h>
+#include <gnustep/xdps/NSDPSContext.h>
+
+@interface PXKCachedImageRep : NSCachedImageRep
+{
+}
+
+@end
 
 @implementation PXKCachedImageRep
 
@@ -38,4 +44,11 @@
   return YES;
 }
 
+- (Class) classForCoder: aCoder
+{
+  if ([self class] == [PXKCachedImageRep class])
+    return [super class];
+  return [self class];
+}
+
 @end
