@@ -27,11 +27,10 @@
 */ 
 
 #include "config.h"
-#include <gnustep/xdps/PXKCursor.h>
-#include <gnustep/xdps/PXKDPSContext.h>
-#include <gnustep/xdps/PXKWindow.h>
+#include <X11/cursorfont.h>
+
+#include <gnustep/xdps/NSDPSContext.h>
 #include <AppKit/NSApplication.h>
-#include <gnustep/xdps/PXKColor.h>
 
 //
 // Backend structure for PXKCursor
@@ -113,7 +112,7 @@ static NSCursor *xdps_hidden_cursor = nil;
   // Create the blank cursor if necessary
   if (!xdps_blank_cursor)
     {
-      Display *xDisplay = [(PXKDPSContext *)[NSDPSContext currentContext]
+      Display *xDisplay = [(NSDPSContext *)[NSDPSContext currentContext]
 					    xDisplay];
       NSApplication *theApp = [NSApplication sharedApplication];
       NSWindow *w = [theApp mainWindow];
@@ -121,19 +120,27 @@ static NSCursor *xdps_hidden_cursor = nil;
       XColor black, white;
       Cursor c;
 
-      xdps_blank_cursor = [[NSCursor alloc] initWithImage: nil];
+      Window win;
 
-      shape = XCreatePixmapFromBitmapData(xDisplay, [(PXKWindow *)w xWindow],
+      if (w)
+	win = [(PXKWindow *)w xWindow];
+      else
+	{
+	  NSDPSContext* context = (NSDPSContext *)[NSDPSContext currentContext];
+	  win = [context xAppRootWindow];
+	}
+      
+      xdps_blank_cursor = [[NSCursor alloc] initWithImage: nil];
+      shape = XCreatePixmapFromBitmapData(xDisplay, win,
 					  xdps_blank_cursor_bits, 
 					  16, 16, 1, 0, 1);
-      mask = XCreatePixmapFromBitmapData(xDisplay, [(PXKWindow *)w xWindow],
+      mask = XCreatePixmapFromBitmapData(xDisplay, win,
 					 xdps_blank_cursor_bits, 
 					 16, 16, 1, 0, 1);
       black = [(PXKColor *)[NSColor blackColor] xColor];
       white = [(PXKColor *)[NSColor whiteColor] xColor];
-
-      c = XCreatePixmapCursor(xDisplay, shape, mask, 
-			      &white, &black, 0, 0);
+	
+      c = XCreatePixmapCursor(xDisplay, shape, mask, &white, &black, 0, 0);
       [(PXKCursor *)xdps_blank_cursor setXCursor: c];
     }
 
@@ -168,7 +175,7 @@ static NSCursor *xdps_hidden_cursor = nil;
 {
   NSCursor *cur;
   Cursor c;
-  Display *xDisplay = [(PXKDPSContext *)[NSDPSContext currentContext]
+  Display *xDisplay = [(NSDPSContext *)[NSDPSContext currentContext]
 					xDisplay];
 
   cur = [[[NSCursor alloc] initWithImage: nil] autorelease];
@@ -184,7 +191,7 @@ static NSCursor *xdps_hidden_cursor = nil;
 {
   NSCursor *cur;
   Cursor c;
-  Display *xDisplay = [(PXKDPSContext *)[NSDPSContext currentContext]
+  Display *xDisplay = [(NSDPSContext *)[NSDPSContext currentContext]
 					xDisplay];
 
   cur = [[[NSCursor alloc] initWithImage: nil] autorelease];
@@ -233,17 +240,34 @@ static NSCursor *xdps_hidden_cursor = nil;
 + (void)currentCursorHasChanged
 {
   NSCursor *c = [self currentCursor];
+  BOOL isValidXCursor = [(PXKCursor *)c xCursorIsValid];
 
-  if ([(PXKCursor *)c xCursorIsValid])
+  if (isValidXCursor)
     {
-      Display *xDisplay = [(PXKDPSContext *)[NSDPSContext currentContext]
+      Display *xDisplay = [(NSDPSContext *)[NSDPSContext currentContext]
 					    xDisplay];
       Cursor xc = [(PXKCursor *)c xCursor];
       NSApplication *theApp = [NSApplication sharedApplication];
       NSWindow *w = [theApp mainWindow];
+      Window win;
+      
+      if (w)
+	win = [(PXKWindow *)w xWindow];
+      else
+	{
+	  NSDPSContext* context =(NSDPSContext *)[NSDPSContext currentContext];
+	  win = [context xAppRootWindow];
+	}
 
-      XDefineCursor(xDisplay, [(PXKWindow *)w xWindow], xc);
+      XDefineCursor(xDisplay, win, xc);
     }
 }
 
+- (Class) classForCoder: aCoder
+{
+  if ([self class] == [PXKCursor class])
+    return [super class];
+  return [self class];
+}
+
 @end
