@@ -25,13 +25,20 @@
 */ 
 
 #include <config.h>
-#include <gnustep/xdps/PXKit.h>
+#include <gnustep/xdps/NSDPSContext.h>
 #include <DPS/dpsXshare.h>
-#ifdef HAVE_DPS_DPSNXARGS_H
+#if HAVE_DPS_DPSNXARGS_H
 #include <DPS/dpsNXargs.h>
 #endif
 
-#if 0
+#include <AppKit/GSWraps.h>
+#include "general.h"
+#include "PXKDrawingEngine.h"
+
+#define CUR_CONTEXT     ((NSDPSContext*)[NSDPSContext currentContext])
+
+void PXKInitializeDrawingEngine(void);
+
 //
 // This functions performs all backend initialization
 // that is required before main() is called
@@ -42,71 +49,40 @@ initialize_gnustep_backend(void)
   // Have backend classes poseAs their super
   NSDebugLog(@"Using GNUstep GUI X/DPS Backend for backend implementation.\n");
 
-  [PXKDPSContext poseAsClass:[NSDPSContext class]];
+  [NSGraphicsContext setDefaultContextClass: [NSDPSContext class]];
   [PXKScreen poseAsClass:[NSScreen class]];
-
   [PXKFontManager poseAsClass:[NSFontManager class]];
   [PXKFont poseAsClass:[NSFont class]];
-
-  [PXKResponder poseAsClass:[NSResponder class]];
   [PXKApplication poseAsClass:[NSApplication class]];
-  [PXKView poseAsClass:[NSView class]];
-
-  [PXKCell poseAsClass:[NSCell class]];
-  [PXKControl poseAsClass:[NSControl class]];
-
-  [PXKFontPanel poseAsClass:[NSFontPanel class]];
-  [PXKMenuItem poseAsClass:[NSMenuItem class]];
-  [PXKMenuMatrix poseAsClass:[NSMenuMatrix class]];
-  [PXKMenu poseAsClass:[NSMenu class]];
-
+  [PXKMenuWindow poseAsClass:[NSMenuWindow class]];
   [PXKColor poseAsClass:[NSColor class]];
 
-  [PXKBox poseAsClass:[NSBox class]];
-  [PXKButtonCell poseAsClass:[NSButtonCell class]];
-  [PXKButton poseAsClass:[NSButton class]];
-
-  [PXKSavePanel poseAsClass:[NSSavePanel class]];
-  [PXKOpenPanel poseAsClass:[NSOpenPanel class]];
-
-  [PXKPopUpButton poseAsClass:[NSPopUpButton class]];
-  [PXKSliderCell poseAsClass:[NSSliderCell class]];
-  [PXKTextFieldCell poseAsClass:[NSTextFieldCell class]];
-  [PXKTextField poseAsClass:[NSTextField class]];
-  [PXKColorWell poseAsClass:[NSColorWell class]];
-  [PXKScroller poseAsClass:[NSScroller class]];
-  [PXKClipView poseAsClass:[NSClipView class]];
-  [PXKScrollView poseAsClass:[NSScrollView class]];
-  [PXKSplitView poseAsClass:[NSSplitView class]];
-
   // image classes
   [PXKImage poseAsClass: [NSImage class]];
-  [PXKImageRep poseAsClass: [NSImageRep class]];
   [PXKBitmapImageRep poseAsClass: [NSBitmapImageRep class]];
   [PXKCachedImageRep poseAsClass: [NSCachedImageRep class]];
-  [PXKCustomImageRep poseAsClass: [NSCustomImageRep class]];
   [PXKEPSImageRep poseAsClass: [NSEPSImageRep class]];
 
   [PXKCursor poseAsClass: [NSCursor class]];
-
-  [PXKBrowserCell poseAsClass: [NSBrowserCell class]];
-
   [PXKWindow poseAsClass:[NSWindow class]];
-  [PXKPanel poseAsClass:[NSPanel class]];
 
   // Initialize the XWindows drawing engine
   NSDebugLog(@"Initializing XWindows drawing engine\n");
-  initializeDrawingEngine();
-  [PXKText poseAsClass:[NSText class]];
+  PXKInitializeDrawingEngine();
 
   /* Force font manager to initialize with the list of fonts available on
      system. */
   [NSFontManager sharedFontManager];
 
+  if (GSDebugSet(@"NSDPSContext") == YES)
+    {
+      NSLog(@"NSDPSContext: Tracing Postscript \n");
+      [(NSDPSContext *)[NSDPSContext currentContext] setOutputTraced: YES];
+    }
+
   // success
   return YES;
 }
-#endif
 
 /*
   cacheSystemImages() loads up any commonly used system images so that
@@ -132,44 +108,37 @@ cacheSystemImages()
  * when the application starts
  */
 
+/* Initialize drawing engine */
 void
-initializeDrawingEngine(void)
+PXKInitializeDrawingEngine(void)
 {
-  char *displayName = NULL;
-  int xscreen;
-  NSRect r;
-  Display *xDisplay;
-
-  if ( (xDisplay = XOpenDisplay(displayName)) == NULL ) {
-    NSLog(@"Cannot connect to X server.  exiting.\n");
-    exit(-1);
-  }
-
-  [(PXKDPSContext *)[NSDPSContext currentContext] setXDisplay: xDisplay];
-
-  [(PXKScreen *)[NSScreen mainScreen] setXScreen: DefaultScreen(xDisplay)];
-  xscreen = [(PXKScreen *)[NSScreen mainScreen] xScreen];
-
-  r = [[NSScreen mainScreen] frame];
-  NSDebugLog(@"Display width: %f height: %f\n", r.size.width, r.size.height);
-
-#if HAVE_DPS_DPSNXARGS_H    
-  /* Make it possible for this client to start a DPS NX agent */
-  XDPSNXSetClientArg(XDPSNX_AUTO_LAUNCH, (void *)True);
-#else
-    if (!XDPSExtensionPresent(xDisplay)) {
-        NSLog (@"DPS extension not in server!");
-        exit (1);
-    }
-#endif
-
-  // create the DPS context
-  [(PXKDPSContext *)[NSDPSContext currentContext] createDPSContext];
-
-  // cache any system images
-//  cacheSystemImages();
+  NSDPSContext *context;
+  context = (NSDPSContext *)[NSDPSContext defaultContextWithInfo: nil];
+  [NSDPSContext setCurrentContext: context];
 }
 
+/* Wraps */
+/* Graphics Wraps */
+void 
+GSWSetMatrix(NSGraphicsContext *ctxt, float m[6])
+{
+  PSWSetMatrix(m);
+}
+
+/* Context helper wraps */
+unsigned int 
+GSWDefineAsUserObj(NSGraphicsContext *ctxt)
+{
+  return PSDefineAsUserObj();
+}
+
+void 
+GSWViewIsFlipped(GSCTXT *ctxt, BOOL flipped)
+{
+}
+
+
+
 /* Private functions, not part of the OpenStep specification */
 NSPoint intersectionPoint(NSRect lineA, NSRect lineB)
 {
@@ -382,3 +351,51 @@ NSRect clipLineToRect(NSRect aLine, NSRect aRect)
 
     return resultLine;
 }
+
+Window _findXWindow(Display* display,
+		    Window topwindow,
+		    Window window_to_check,
+		    int x, int y)
+{
+  int		newx, newy;
+  Window	window;
+
+  // find the lowest X window in the heirarchy that the global x,y
+  // coordinates are in.
+  if (topwindow == (Window) None) 
+    {
+      fprintf(stderr,"_findXWindow() error: topwindow == (Window) None \n");
+
+      return (Window) None;
+    }
+
+  if (window_to_check == (Window) None)
+    {
+      fprintf(stderr,
+	"_findXWindow() error: window_to_check == (Window) None \n");
+
+      return topwindow;
+    }
+
+  // find lowest X window by iterating thru the X window's heirarchy while
+  // XTranslateCoordinates returns an X Window via the window ptr
+
+  window = window_to_check;
+  while ((XTranslateCoordinates(display, topwindow, window_to_check,
+	    x, y, &newx, &newy, &window) != 0)
+	    && (window != (Window) None))
+    {
+      if (window != (Window) None)
+	{
+	  topwindow = window_to_check;
+	  window_to_check = window;
+	  x = newx;
+	  y = newy;
+	}
+    }
+
+  if (window == (Window) None)
+    window = window_to_check;
+
+  return window;
+}
