@@ -28,31 +28,13 @@
 
 #include "config.h"
 #include <AppKit/NSColor.h>
-#include <gnustep/xdps/PXKImage.h>
-#include <gnustep/xdps/PXKView.h>
-#include <gnustep/xdps/PXKScreen.h>
-#include <gnustep/xdps/PXKDPSContext.h>
-#include <gnustep/xdps/PXKWindow.h>
-#include <gnustep/xdps/PXKCachedImageRep.h>
+#include <gnustep/xdps/NSDPSContext.h>
+
+@interface PXKImage : NSImage
+@end
 
 @implementation PXKImage
 
-- (void) lockFocusOnRepresentation: (NSImageRep *)imageRep
-{
-  [super lockFocusOnRepresentation: imageRep];
-
-  // Save the graphics state
-  PSgsave();
-}
-
-- (void) unlockFocus
-{
-  [super unlockFocus];
-
-  // Restore the graphics state
-  PSgrestore();
-}
-
 - (BOOL)drawRepresentation: (NSImageRep *)imageRep inRect: (NSRect)rect
 {
   // Draw the background color
@@ -71,4 +53,11 @@
   return [super drawRepresentation: imageRep inRect: rect];
 }
 
+- (Class) classForCoder: aCoder
+{
+  if ([self class] == [PXKImage class])
+    return [super class];
+  return [self class];
+}
+
 @end
